#include "battery.h"
#include "bulletml-tinyxml.h"
#include "enemy.h"
#include "game.h"
#include "scroll.h"
#include "graphics.h"

#include <boost/smart_ptr.hpp>

const double INVALID_DIR = 1000;

Battery::Battery(const Point& pnt, const std::string& xml)
	: pnt_(pnt), dir_(180), turn_(0), col_(WHITE), rank_(0.5),
	  isLaser_(false), isScroll_(true), width_(0), enemy_(0), alive_(true),
	  invisible_(0), childColor_(WHITE), dirPrev_(INVALID_DIR)
{
	bulletml_.reset(new BulletMLTinyXML(
						"data/"+Game::instance()->bulletDir()+"/"+xml));
	bulletml_->Parse();
}

Battery::~Battery() {}

void Battery::event() {
	if (!alive_) return;

	Game* game = Game::instance();

	if (game->turn() >= turn_ && enemy_ == 0) {
		Point p = game->scroll()->convertPoint(pnt_);
		if (p.y > 0) {
			assert(childColor_ != EMPTY);
			enemy_ = new Enemy(p, Graphics::instance()->enemyBoss(col_),
							   bulletml_.get(), col_, rank_, dir_, invisible_,
							   childColor_, isLaser_, width_);
			Game::instance()->addEnemy(boost::shared_ptr<Enemy>(enemy_));
		}
	}

	if (enemy_ != 0) {
		if (isScroll_) {
			enemy_->setPnt(game->scroll()->convertPoint(pnt_));
			if (game->scroll()->isRoll()) {
				double r = game->scroll()->convertDir(dtor(dir_));
				enemy_->setDir(r);
				if (enemy_->isLaser()) {
					if (dirPrev_ != INVALID_DIR) {
						enemy_->updateRollLaser(r - dirPrev_);
					}
					dirPrev_ = r;
				}
			}
			if (enemy_->isLaser()) {
				enemy_->updateLaser();
			}
		}
	}

	if (enemy_ != 0 && !enemy_->alive()) {
		alive_ = false;
	}
}
