#include "config.h"

#include "bulletml.h"
#include "bulletml-tinyxml.h"
#include "enemy.h"

#include <vector>
#include <algorithm>
#include <functional>
#include <string>
#include <cassert>

BulletMLTinyXML::BulletMLTinyXML(const std::string& filename)
    : xmlFile_(filename), curNode_(0)
{}

void BulletMLTinyXML::getTree(TiXmlNode* node) {
    translateNode(node);

    TiXmlNode* child;
    for (child = node->FirstChild(); child; child = child->NextSibling()) {
		TiXmlText* text;
		if ((text = child->ToText()) != 0) {
			curNode_->setValue(text->Value());
			break;
		}

		getTree(child);
    }

    curNode_ = curNode_->getParent();
}

void BulletMLTinyXML::translateNode(TiXmlNode* node) {
    TiXmlElement* elem = node->ToElement();
    assert(elem != 0);

    XMLNode* xmlNode = addContent(elem->Value());

    MyAttributes mattr;
    TiXmlAttribute* attr;
    for (attr = elem->FirstAttribute(); attr; attr = attr->Next()) {
		mattr.push_back(attr->Name());
		mattr.push_back(attr->Value());
    }
    addAttribute(mattr, xmlNode);

    checkNode(xmlNode);

    if (curNode_ != 0) curNode_->addChild(xmlNode);
    curNode_ = xmlNode;
}

void BulletMLTinyXML::Parse() {
    TiXmlDocument doc(xmlFile_);
    doc.LoadFile();

    TiXmlNode* node;
    for (node = doc.FirstChild(); node; node = node->NextSibling()) {
		if (node->ToElement() != 0) {
			getTree(node);
			break;
		}
    }
}

