#include "command.h"
#include "game.h"
#include "util.h"
#include "conf.h"
#include "player.h"
#include "bulletml.h"
#include "shot.h"

extern "C" {
#include "calc.h"
}

#include <fstream>
#include <sstream>
#include <algorithm>
#include <functional>

#include <cmath>
#include <cctype>

#include <boost/smart_ptr.hpp>

const double EnemyCommand::speedRatio = 8.5333333;

EnemyCommandMap EnemyCommand::commandMap_;

double EnemyCommand::getDirection(XMLNode* dirNode) {
    double dir;

    bool isDefault = true;
    dir = getNumberContents(dirNode);

    if (dirNode->haveAttribute("type")) {
		const std::string& type = dirNode->getAttribute("type");
		isDefault = false;

		if (type == "absolute") {
			if (bulletml_->isHorizontal()) {
				dir -= 90;
			}
		}
		else if (type == "relative") {
			dir += rtod(enemy_->polar_.r);
		}
		else if (type == "sequence") {
			if (!prevDir_.isValidate()) {
				dir = 0;
				isDefault = true;
			}
			else dir += prevDir_;
		}
		else {
			isDefault = true;
		}
    }

    if (isDefault) {
		// 餫Ψ
		dir += rtod(Game::instance()->player()->center().angle(
						enemy_->center()));
    }

    while (dir > 360) dir -= 360;
    while (dir < 0) dir += 360;

    prevDir_ = dir;

    return dtor(dir);
}

double EnemyCommand::getSpeed(XMLNode* spdNode) {
    double spd;

    spd = getNumberContents(spdNode);
    if (spdNode->haveAttribute("type")) {
		const std::string& type = spdNode->getAttribute("type");

		if (type == "relative") {
			spd += enemy_->polar_.l / speedRatio;
		}
		else if (type == "sequence") {
			if (!prevSpd_.isValidate()) spd = 1;
			else spd += prevSpd_;
		}
    }

    // ǥեȤǤϲ⤷ʤ

    prevSpd_ = spd;

    return spd * speedRatio;
}

PolarCoord EnemyCommand::getNewShotSpeed() {
    if (!spd_.isValidate()) {
        spd_ = speedRatio;
        prevSpd_ = 1;
    }
    if (!dir_.isValidate()) {
		dir_ = Game::instance()->player()->center().angle(enemy_->center());
        prevDir_ = rtod(dir_);
    }
    return PolarCoord(spd_, dir_);
}

void EnemyCommand::createShot() {
	boost::shared_ptr<Shot> shot(
		new Shot(enemy_->center(), Point(getNewShotSpeed()),
				 enemy_->childColor_, enemy_->isLaser_,
				 enemy_->width_, enemy_->spd_));
    Game::instance()->addShot(shot);

	if (shot->isLaser()) {
		enemy_->children_.push_back(shot);
	}
}

void EnemyCommand::createEnemy() {
    std::vector<XMLNode*> acts;
    act_->getAllChildren("action", std::back_inserter(acts));
    act_->getAllChildren("actionRef", std::back_inserter(acts));
	Enemy* enemy = new Enemy(
		enemy_->center(), getNewShotSpeed(), bulletml_, acts,
		parameters_.get(), enemy_->col_,
		enemy_->rank_, enemy_->invisible_-1,
		enemy_->childColor_, enemy_->isLaser_, enemy_->width_);
	Game::instance()->addEnemy(boost::shared_ptr<Enemy>(enemy));
}

EnemyCommand::EnemyCommand(BulletML* command, XMLNode* node,
						   Enemy* enemy, const std::vector<double>* para)
    : enemy_(enemy), bulletml_(command), act_(node)
{
    actTurns_ = Game::instance()->turn();
    parameters_.reset(new std::vector<double>);
    if (para != 0)
		std::copy(para->begin(), para->end(),
				  std::back_inserter(*parameters_));
}

EnemyCommand::~EnemyCommand() {
}

double EnemyCommand::getNumberContents(const XMLNode* node) {
    assert(node);

    std::string str = node->getValue();

    //  calc Ǥ٤ѿѴ
    size_t dollar;
    while ((dollar = str.find('$')) != std::string::npos) {
		std::ostringstream oss;
		if (str.substr(dollar, 5) == "$rand") {
			oss << rnd(1.0);
			str.replace(dollar, 5, oss.str());
		}
		else if (str.substr(dollar, 5) == "$rank") {
			oss << enemy_->rank_;
			str.replace(dollar, 5, oss.str());
		}
		else {
			std::istringstream iss(str.substr(dollar+1, 1));
			int num;
			iss >> num;
			oss << (*parameters_)[num];
			str.replace(dollar, 2, oss.str());
		}
    }

    str += '\n';
    return calc(str.c_str());
}

void EnemyCommand::setSpeed() {
    XMLNode* spd = act_->getChild("speed");
    if (spd == 0) return;

    spd_ = getSpeed(spd);
}

void EnemyCommand::setDirection() {
    XMLNode* dir = act_->getChild("direction");
    if (dir == 0) return;

    dir_ = getDirection(dir);
}

bool EnemyCommand::isTickEnd() const {
    return Game::instance()->turn() <= actTurns_;
}

bool EnemyCommand::isTurnEnd() const {
    return (isEnd() || isTickEnd());
}

bool EnemyCommand::isEnd() const {
    return act_ == 0 || !enemy_->alive();
}

void EnemyCommand::init() {
    try {
		act_ = bulletml_->getActionRef("top");
    }
    catch (std::runtime_error re) {
		std::cerr << "<action label=\"top\"> hasn't come." << std::endl;
		throw re;
    }

    // 롼
    act_->setParent(0);
}

void EnemyCommand::run() {
    // ͤ˥ɤ
    while (!isTurnEnd()) {
		XMLNode* prev = act_;
		Method fp = commandMap_.getMethod(act_->getName());
		(this->*fp)();

		if (act_ == 0 &&
			prev->getParent() != 0 &&
			prev->getParent()->getName() == "bulletml")
		{
			ASSERT(!refStack_.empty(), "Many Refs.");
			prev = refStack_.top().first;
			parameters_.reset(refStack_.top().second);
			refStack_.pop();
		}

		if (act_ == 0) act_ = prev->next();

		while (act_ == 0) {
			if (prev->getParent() != 0 &&
				prev->getParent()->getName() == "repeat")
			{
				std::pair<int, int>& rep = repeatStack_.top();
				rep.first++;
				if (rep.first < rep.second) {
					act_ = prev;
					break;
				}
				else {
					repeatStack_.pop();
				}
			}

			act_ = prev->getParent();
			if (act_ == 0) break;

			prev = act_;

			if (prev->getParent() != 0 &&
				prev->getParent()->getName() == "bulletml")
			{
				ASSERT(!refStack_.empty(), "Many Refs.");
				prev = act_ = refStack_.top().first;
				parameters_.reset(refStack_.top().second);
				refStack_.pop();
			}

			act_ = act_->next();
		}
    }
}

void EnemyCommand::runBullet() {
    setSpeed();
    setDirection();

    if (act_->getChild("action") == 0 && act_->getChild("actionRef") == 0) {
		createShot();
    }
    else {
		createEnemy();
    }

    act_ = 0;
}

void EnemyCommand::runFire() {
    shotInit();

    setSpeed();
    setDirection();

    XMLNode* bullet = act_->getChild("bullet");
    if (bullet == 0) bullet = act_->getChild("bulletRef");
    ASSERT(bullet != 0, "fire elem must have contents bullet or bulletRef");

    act_ = bullet;
}

void EnemyCommand::runAction() {
    act_ = *act_->childBegin();
}

void EnemyCommand::runWait() {
    int waitTurn = static_cast<int>(getNumberContents(act_));
    actTurns_ += waitTurn;

    act_ = 0;
}

void EnemyCommand::runRepeat() {
    const XMLNode* times = act_->getChild("times");
    if (times == 0) return;

    int timesNum = static_cast<int>(getNumberContents(times));

    repeatStack_.push(std::make_pair(0, timesNum));

    XMLNode* action = act_->getChild("action");
    if (action == 0) action = act_->getChild("actionRef");
    ASSERT(action != 0, "repeat elem must have contents action or actionRef");

    act_ = action;
}

void EnemyCommand::runFireRef() {
    std::vector<double>* prevPara = parameters_.release();
    parameters_.reset(getParameters());

    const std::string refTo = act_->getAttribute("label");
    refStack_.push(make_pair(act_, prevPara));
    act_ = bulletml_->getFireRef(refTo);
}

void EnemyCommand::runActionRef() {
    std::vector<double>* prevPara = parameters_.release();
    parameters_.reset(getParameters());

    const std::string refTo = act_->getAttribute("label");
    refStack_.push(make_pair(act_, prevPara));
    act_ = bulletml_->getActionRef(refTo);
}

void EnemyCommand::runBulletRef() {
    std::vector<double>* prevPara = parameters_.release();
    parameters_.reset(getParameters());

    const std::string refTo = act_->getAttribute("label");
    refStack_.push(make_pair(act_, prevPara));
    act_ = bulletml_->getBulletRef(refTo);
}

void EnemyCommand::runChangeDirection() {
    int term = static_cast<int>(getNumberContents(act_->getChild("term")));
    int finalTurn = actTurns_ + term;

    double dirFirst = enemy_->polar_.r;
    double dir, dirSpace;

    XMLNode* dirNode = act_->getChild("direction");
    // direction  type  sequence λư
    if (dirNode->haveAttribute("type") &&
		dirNode->getAttribute("type") == "sequence") {
		dirSpace = dtor(getNumberContents(dirNode)) * term;
		dir = dirFirst + dirSpace;
    }
    else {
		dir = getDirection(dirNode);

		// ȶᤤäƤΤϷ빽񤷤
		double dirSpace1 = dir - dirFirst;
		double dirSpace2;
		if (dirSpace1 > 0) dirSpace2 = dirSpace1 - PI * 2;
		else dirSpace2 = dirSpace1 + PI * 2;
		if (std::abs(dirSpace1) < std::abs(dirSpace2)) dirSpace = dirSpace1;
		else dirSpace = dirSpace2;
    }

    enemy_->changeDir_ = new LinearFunc<int, double>
		(actTurns_, finalTurn, dirFirst, dirFirst + dirSpace);

    act_ = 0;
}

void EnemyCommand::runChangeSpeed() {
	XMLNode* spdNode = act_->getChild("speed");

    double spd;
    int term = static_cast<int>(getNumberContents(act_->getChild("term")));

    // speed  type  sequence λư
    if (spdNode->haveAttribute("type") &&
		spdNode->getAttribute("type") == "sequence") {
		double spdSpace = getNumberContents(spdNode) * term * speedRatio;
		spd = enemy_->polar_.l + spdSpace;
    }
    else {
		spd = getSpeed(spdNode);
    }

    int finalTurn = actTurns_ + term;

    enemy_->changeSpeed_ = new LinearFunc<int, double>
		(actTurns_, finalTurn, enemy_->polar_.l, spd);

    act_ = 0;
}

void EnemyCommand::runAccel() {
	enemy_->spd_ = Point(enemy_->polar_);

    XMLNode* horizontal = act_->getChild("horizontal");
    XMLNode* vertical = act_->getChild("vertical");

    double hAccel, vAccel;
    if (horizontal == 0) hAccel = 0;
    else hAccel = getNumberContents(horizontal)*speedRatio;
    if (vertical == 0) vAccel = 0;
    else vAccel = getNumberContents(vertical)*speedRatio;

    if (bulletml_->isHorizontal()) {
		std::swap(hAccel, vAccel);
		vAccel = -vAccel;
    }

    int term = static_cast<int>(getNumberContents(act_->getChild("term")));

    int finalTurn = actTurns_ + term;

    Point finalSpd = enemy_->spd_ + Point(hAccel, vAccel);

    enemy_->accel_ = new LinearFunc<int, Point>
		(actTurns_, finalTurn, enemy_->spd_, finalSpd);

    act_ = 0;
}

void EnemyCommand::runVanish() {
    enemy_->alive_ = false;
    act_ = 0;
}

std::vector<double>* EnemyCommand::getParameters() {
    std::vector<double>* para = new std::vector<double>;
    // 0ǤϻȤʤ
    para->push_back(0);

    XMLNode::ChildIterator ite;
    for (ite = act_->childBegin(); ite != act_->childEnd(); ite++) {
		XMLNode* node = *ite;
		if (node->getName() != "param") continue;

		para->push_back(getNumberContents(node));
    }

    return para;
}

EnemyCommandMap::EnemyCommandMap() {
    commandMap_["bullet"] = &EnemyCommand::runBullet;
    commandMap_["action"] = &EnemyCommand::runAction;
    commandMap_["fire"] = &EnemyCommand::runFire;
    commandMap_["wait"] = &EnemyCommand::runWait;
    commandMap_["repeat"] = &EnemyCommand::runRepeat;
    commandMap_["bulletRef"] = &EnemyCommand::runBulletRef;
    commandMap_["actionRef"] = &EnemyCommand::runActionRef;
    commandMap_["fireRef"] = &EnemyCommand::runFireRef;
    commandMap_["changeDirection"] = &EnemyCommand::runChangeDirection;
    commandMap_["changeSpeed"] = &EnemyCommand::runChangeSpeed;
    commandMap_["accel"] = &EnemyCommand::runAccel;
    commandMap_["vanish"] = &EnemyCommand::runVanish;
}

bool EnemyCommandMap::isRunnable(const std::string& key) const {
    return commandMap_.end() != commandMap_.find(key);
}

EnemyCommand::Method EnemyCommandMap::getMethod(const std::string& key) const {
    CommandMap_::const_iterator ite = commandMap_.find(key);
    if (ite == commandMap_.end()) ASSERT(0, key + " can't runnable.");
    return ite->second;
}



