#ifndef BOHA_COMMAND
#define BOHA_COMMAND

#include "singleton.h"

#include <vector>
#include <string>
#include <map>
#include <stack>
#include <utility>
#include <memory>

class Enemy;
class EnemyCommand;
class EnemyCommandMap;
class BulletML;
class XMLNode;
class Player;

class EnemyCommand {
public:
    EnemyCommand(BulletML* command, XMLNode* node,
		 Enemy* enemy, const std::vector<double>* para);
    ~EnemyCommand();

    void init();
    void run();

    bool isTickEnd() const;
    bool isTurnEnd() const;
    bool isEnd() const;

public:
    typedef void (EnemyCommand::*Method)();

    void setDirection();
    void setSpeed();

    void createShot();
    void createEnemy();
    void shotInit() {
		spd_.disValidate();
		dir_.disValidate();
    }

protected:
    void runBullet();
    void runAction();
    void runFire();
    void runWait();
    void runRepeat();
    void runBulletRef();
    void runActionRef();
    void runFireRef();
    void runChangeDirection();
    void runChangeSpeed();
    void runAccel();
    void runVanish();

protected:
    double getNumberContents(const XMLNode* node);
    std::vector<double>* getParameters();
    double getSpeed(XMLNode* spdNode);
    double getDirection(XMLNode* dirNode);
    PolarCoord getNewShotSpeed();

protected:
    static EnemyCommandMap commandMap_;

protected:
    Validatable<double> spd_, dir_, prevSpd_, prevDir_;

    typedef std::vector<double> Parameters;
    std::auto_ptr<Parameters> parameters_;

protected:
    Enemy* enemy_;

    BulletML* bulletml_;
    XMLNode* act_;

    int actTurns_;
    bool turnEnd_;

private:
    // ʤȤҤɤ
	static const double speedRatio;

protected:
    typedef std::stack<std::pair<int, int> > RepeatStack;
    RepeatStack repeatStack_;
    typedef std::stack<std::pair<XMLNode*, Parameters*> > RefStack;
    RefStack refStack_;

    friend class EnemyCommandMap;
};

class EnemyCommandMap {
public:
    typedef EnemyCommand::Method Method;

    EnemyCommandMap();

    bool isRunnable(const std::string& key) const;
    Method getMethod(const std::string& key) const;

private:
    typedef std::map<std::string, Method> CommandMap_;
    CommandMap_ commandMap_;
};

#endif // ! BOHA_COMMAND

