#ifndef BOHA_CONF
#define BOHA_CONF

#include "singleton.h"

#include <string>
#include <map>
#include <cassert>

class Conf : public Singleton<Conf, CanNewPolicy> {
public:
    Conf(int argc, char* argv[]);

    typedef std::map<int, std::string> IdToAct;
    typedef enum {CONF, ACT, XML} HowToRun;

    const std::string& scoreFile() const { return scoreFile_; }
    const std::string& actionFile() const { return actionFile_; }
    int enemys() const { return enemys_; }
    int verbose() const { return verbose_; }
    const std::string& command() const { return command_; }
    bool watch() const { return watch_; }
    const std::string& name() const { return name_; }
    const IdToAct& enemyActs() const { return enemyActs_; }
    const std::string& idToAct(int id) const {
	return enemyActs_.find(id)->second;
    }
    const IdToAct& globalActs() const { return globalActs_; }
    const std::string& idToGlobalAct(int id) const {
	return globalActs_.find(id)->second;
    }
    int firstGlobal() const { return firstGlobal_; }
    bool isFastMode() const { return fastMode_; }
    bool isProfiling() const { return profiling_; }
    bool isSlow() const { return isSlow_; }
    double slow() const { return slow_; }
    int level() const { return level_; }
    bool isCpu() const { return cpu_; }

    bool isDefault() const { return default_; }
    HowToRun gameType() const { return howToRun_; }

    void saveNameCache(const std::string& str) const;

private:
    void loadConfFile(std::istream& is);
    void loadName();
    void parseLongOption(const std::string& str) const;
    void help() const;
    void version() const;

    std::string scoreFile_;
    std::string actionFile_;
    int enemys_;
    int verbose_;
    std::string command_;
    bool watch_;
    std::string name_;
    IdToAct enemyActs_;
    IdToAct globalActs_;
    int firstGlobal_;
    bool fastMode_;
    bool profiling_;
    bool isSlow_;
    double slow_;
    int level_;
    bool cpu_;

    bool default_;
    HowToRun howToRun_;

    static const std::string nameCacheFile_;
};

#endif // BOHA_CONF
