#include "enemy.h"
#include "conf.h"
#include "game.h"
#include "bulletml.h"
#include "bulletml-tinyxml.h"
#include "command.h"
#include "player.h"
#include "graphics.h"
#include "builtinfuncs.h"
#include "shot.h"

#include <fstream>
#include <sstream>

const double INVALID_DIR = 1000;

Enemy::Enemy (const Point& pnt, Window* graph,
			  BulletML* bulletml,
			  CharColor col, double rank, double degree, int inv,
			  CharColor childColor, bool isLaser, int width)
    : Charactor(pnt, graph, col),
      changeDir_(0), changeSpeed_(0), accel_(0), rank_(rank), invisible_(inv),
	  childColor_(childColor), isLaser_(isLaser), laserDir_(INVALID_DIR)
{
	width_ = width;
	polar_ = PolarCoord(0, dtor(degree));

	zako_ = false;
	bulletml_ = bulletml;

	if (bulletml_->haveActionRef("top")) {
		XMLNode* act = bulletml_->getActionRef("top");
		act->setParent(0);
		coms_.push_back(new EnemyCommand(bulletml_, act, this, 0));
	}
	// top1, top2 ... 
	else {  
		multiRun(bulletml_);
	}
}

Enemy::Enemy (const Point& pnt, const PolarCoord& spd,
			  BulletML* bulletml,
			  const std::vector<XMLNode*>& acts,
			  const std::vector<double>* para, CharColor col,
			  double rank, int inv, CharColor childColor,
			  bool isLaser, int width)
    : Charactor(pnt, 0, col),
      changeDir_(0), changeSpeed_(0), accel_(0), rank_(rank), invisible_(inv),
	  childColor_(childColor), isLaser_(isLaser)
{
	width_ = width;
    para_.reset(new std::vector<double>);
    std::copy(para->begin(), para->end(), std::back_inserter(*para_));
    bulletml_ = bulletml;
    std::copy(acts.begin(), acts.end(), std::back_inserter(acts_));

    polar_ = spd;

    actIte_ = acts_.begin();

    std::vector<XMLNode*>::const_iterator ite;
    for (ite = acts.begin(); ite != acts.end(); ite++) {
		const XMLNode* act = *ite;
		if (act->findNode("fire")) break;
		if (act->findNode("fireRef")) break;
		if (act->getName() == "actionRef") break;
    }

    if (ite != acts.end()) {
		zako_ = false;
		graph_ = Graphics::instance()->enemyZako(col);
    }
    else {
		zako_ = true;
		graph_ = Graphics::instance()->enemyBullet(col);
    }

	pnt_ = pnt - halfSize();
	cen_ = pnt;

}

void Enemy::moveValue() {
    int now = Game::instance()->turn();

    if (changeDir_ != 0) {
		if (changeDir_->isLast(now)) {
			changeDir_->setLast(polar_.r);
			delete changeDir_;
			changeDir_ = 0;
		}
		else {
			polar_.r = changeDir_->getValue(now);
		}
    }

    if (changeSpeed_ != 0) {
		if (changeSpeed_->isLast(now)) {
			changeSpeed_->setLast(polar_.l);
			delete changeSpeed_;
			changeSpeed_ = 0;
		}
		else {
			polar_.l = changeSpeed_->getValue(now);
		}
    }

    if (accel_ != 0) {
		if (accel_->isLast(now)) {
			accel_->setLast(spd_);
			delete accel_;
			accel_ = 0;
			polar_ = PolarCoord(spd_);
		}
		else {
			spd_ = accel_->getValue(now);
		}
    }
	else {
		spd_ = Point(polar_);
	}

    Charactor::moveValue();
}

void Enemy::moveGraphic() const {
	if (!isInvisible())
		surface_->drawLine(pnt_, spd_*4, width_, col_.color(), col_.color2());
}

Enemy::~Enemy() {
    delete changeDir_;
    delete changeSpeed_;
    delete accel_;
	typedef std::deque<boost::shared_ptr<Shot> > Children;

	// 졼ξõ
	for (Children::iterator ite = children_.begin();
		 ite != children_.end(); ++ite)
	{
		(*ite)->die();
	}
}

void Enemy::multiRun(BulletML* bulletml) {
    std::vector<XMLNode*> acts;
    int i = 1;
    while (1) {
		std::ostringstream oss;
		oss << i;
		std::string topNum = "top" + oss.str();
		if (!bulletml->haveActionRef(topNum)) break;
		XMLNode* act = bulletml->getActionRef(topNum);
		act->setParent(0);
		acts.push_back(act);
		i++;
    }

    std::vector<XMLNode*>::iterator actIte;
    for (actIte = acts.begin(); actIte != acts.end(); actIte++) {
		coms_.push_back(new EnemyCommand(bulletml, *actIte, this, 0));
    }
}

void Enemy::huntEvent() {
    if (!coms_.empty() &&
		std::find_if(coms_.begin(), coms_.end(),
					 std::not1(std::mem_fun(&EnemyCommand::isEnd)))
		== coms_.end())
    {
		// Ǹ wait ̿Ԥ
		if (std::find_if(coms_.begin(), coms_.end(),
						 std::mem_fun(&EnemyCommand::isTickEnd))
			!= coms_.end())
		{
			return;
		}

		delete_clear(coms_);
    }

    if (coms_.empty() && !acts_.empty()) {
		if (actIte_ == acts_.end()) return;

		XMLNode* act = *actIte_;
		act->setParent(0);
		coms_.push_back(new EnemyCommand(bulletml_, act, this, para_.get()));

		actIte_++;
    }

    std::for_each(coms_.begin(), coms_.end(),
				  std::mem_fun(&EnemyCommand::run));
}

void Enemy::hitByPlayer (Player*) {
}

void Enemy::hitByEnemy (Enemy*) {
}

void Enemy::hitByShot (Shot*) {
}

void Enemy::updateLaser() {
	typedef std::deque<boost::shared_ptr<Shot> > Children;

	// 졼ˤΰư
	for (Children::iterator ite = children_.begin();
		 ite != children_.end(); ++ite)
	{
		Shot* shot = ite->get();
		if (shot->alive())
			shot->setPnt(pnt_ + shot->spd() * shot->count());
	}
}

void Enemy::updateRollLaser(double r) {
	if (children_.empty()) return;
	while (!children_.front()->alive()) {
		children_.pop_front();
		if (children_.empty()) return;
	}

	if (laserDir_ == INVALID_DIR) {
		PolarCoord ps(children_.front()->spd());
		laserDir_ = ps.r;
	}

	laserDir_ += r;

	typedef std::deque<boost::shared_ptr<Shot> > Children;

	// 졼ˤβž
	for (Children::iterator ite = children_.begin();
		 ite != children_.end(); ++ite)
	{
		Shot* shot = ite->get();
		if (shot->alive()) {
			PolarCoord ps(shot->spd());
			ps.r = laserDir_;
			shot->setSpd(Point(ps));
		}
	}
}
