#ifndef BOHA_ENEMY
#define BOHA_ENEMY

#include "util.h"
#include "char.h"

#include <vector>
#include <map>
#include <memory>
#include <deque>

#include <SDL.h>

#include <boost/smart_ptr.hpp>

class EnemyCommand;
class BulletML;
class XMLNode;

/// ŨΥ饹
class Enemy : public Charactor {
public:
    Enemy (const Point& pnt, Window* graph,
		   BulletML* bulletml,
		   CharColor col, double rank, double degree, int inv,
		   CharColor childColor, bool isLaser, int width);
    Enemy (const Point& pnt, const PolarCoord& spd,
		   BulletML* bulletml,
		   const std::vector<XMLNode*>& acts, const std::vector<double>* para,
		   CharColor col, double rank, int inv,
		   CharColor childColor, bool isLaser, int width);
    virtual ~Enemy();

    void huntEvent ();
    void multiRun(BulletML* bulletml);

    virtual void hit (Charactor* bc) { bc->hitByEnemy(this); }
    virtual void hitByPlayer (Player* bc);
    virtual void hitByEnemy (Enemy* bc);
    virtual void hitByShot (Shot* bc);

    virtual void moveValue();
	virtual void moveGraphic() const;

	double getRank() const { return rank_; }

	virtual void setSpd(const Point& spd) {
		polar_ = PolarCoord(spd);
	}
	virtual void setDir(double r) {
		polar_.r = r;
	}

	bool isInvisible() const { return invisible_ > 0; }
	virtual bool isLaser() const { return isLaser_; }

	void updateLaser();
	void updateRollLaser(double r);

private:
    std::vector<EnemyCommand*> coms_;

    std::vector<XMLNode*> acts_;
    std::vector<XMLNode*>::iterator actIte_;
    BulletML* bulletml_;
    std::auto_ptr<std::vector<double> > para_;

    PolarCoord polar_;
    LinearFunc<int, double>* changeDir_;
    LinearFunc<int, double>* changeSpeed_;
    LinearFunc<int, Point>* accel_;

	double rank_;

	int invisible_;
	CharColor childColor_;
	bool isLaser_;
	double laserDir_;

	/// 졼Ƥ
	std::deque<boost::shared_ptr<Shot> > children_;

    // ޥɤμºݤ¤ϰѾƤޤ
    friend class EnemyCommand;
    friend int enemyLoop(void*);
};

#endif // ! BOHA_ENEMY
