/// Ϥʤ
/**
 * ܡɤȥ祤ƥå
 * ȥܥ󤬿ĤǹϤʤ
 */

#ifndef __MYSDL_PADINPUT_H__
#define __MYSDL_PADINPUT_H__

#include "mysdl_error.h"
#include "mysdl_joystick.h"

#include <singleton.h>

#include <SDL.h>

#include <vector>
#include <memory>
#include <utility>
#include <map>

namespace MSDL {

	class Axis {
	public:
		static const int NONE;
		static const int UP;
		static const int UPRIGHT;
		static const int RIGHT;
		static const int DOWNRIGHT;
		static const int DOWN;
		static const int DOWNLEFT;
		static const int LEFT;
		static const int UPLEFT;

	public:
		Axis() : _axis(NONE) {}
		explicit Axis(int axis) : _axis(axis) {}

	public:
		void add(int axis) { _axis |= axis; }
		void add(Axis axis) { add(axis._axis); }

	public:
		std::string getName() const;

	public:
		int getAxisCode() const { return _axis; }

	public:
		bool operator == (Axis rhs) { return _axis == rhs._axis; }
		bool operator != (Axis rhs) { return _axis != rhs._axis; }
		bool operator == (int rhs) { return _axis == rhs; }
		bool operator != (int rhs) { return _axis != rhs; }
		friend inline bool operator == (int lhs, Axis rhs) {
			return rhs == lhs;
		}
		friend inline bool operator != (int lhs, Axis rhs) {
			return rhs != lhs;
		}
		
		Axis& operator = (int rhs) {
			_axis = rhs;
			return *this;
		}

	public:
		bool isRight() const {
			return _axis & 4;
		}
		bool isLeft() const {
			return _axis & 8;
		}
		bool isDown() const {
			return _axis & 2;
		}
		bool isUp() const {
			return _axis & 1;
		}
		bool isXAxis() const {
			return isRight() || isLeft();
		}
		bool isYAxis() const {
			return isUp() || isDown();
		}

	private:
		int _axis;
	};

    /// ܥϴ쥯饹
	class ButtonInput {
	public:
		virtual bool getButton() =0;
		virtual std::string getName() const =0;
	};

    /// ܡɤˤܥϥ饹
	class KeyButtonInput : public ButtonInput {
	public:
		KeyButtonInput(SDLKey key) : _key(key) {}
		virtual bool getButton() { return SDL_GetKeyState(0)[_key]; }

		virtual std::string getName() const;

	private:
		SDLKey _key;
	};

    /// 祤ƥåˤܥϥ饹
	class JoystickButtonInput : public ButtonInput {
	public:
		JoystickButtonInput(int index, int button)
			: _joystick(JoystickMgr::instance()->getJoystick(index)),
		  _button(button) {
			checkError(_joystick != 0,
					   std::string(
						   "JoystickButtonInput::JoystickButtonInput:") +
					   "bad joystick index.");
		}
		virtual bool getButton() {
			return _joystick->getButton(_button); 
		}

		virtual std::string getName() const;

	private:
		Joystick* _joystick;
		int _button;
	};

    /// 祤ƥåܥˤܥϥ饹
	class JoystickAxisButtonInput : public ButtonInput {
	public:
		static const int JOYSTICK_PLAY_WIDTH;

	public:
		/**
		 * @param axis ФꤷƤϤʤʤ
		 */
		JoystickAxisButtonInput(int index, Axis axis)
			: _joystick(JoystickMgr::instance()->getJoystick(index)),
           _axis(axis)
			{
				checkError(_joystick != 0,
						   std::string("JoystickAxisButtonInput::JoystickAxisButtonInput: bad joystick index."));
					   }
		virtual bool getButton();

		virtual std::string getName() const;
	public:
		/// SDL ɽ MSDL Ѵ
		static Axis generateAxis(Uint8 axis, Sint16 value);

	private:
		Joystick* _joystick;
		Axis _axis;
	};

	class AxisInput {
	public:
		virtual Axis getAxis() const =0;
		virtual ~AxisInput() {}
	};

    /// ButtonInput ˤϥ饹
	class ButtonAxisInput : public AxisInput {
	public:
		ButtonAxisInput() {}
		ButtonAxisInput(std::auto_ptr<ButtonInput> up,
						std::auto_ptr<ButtonInput> right,
						std::auto_ptr<ButtonInput> down,
						std::auto_ptr<ButtonInput> left)
			{
				_up.push_back(up.release());
				_right.push_back(right.release());
				_down.push_back(down.release());
				_left.push_back(left.release());
			}
		virtual ~ButtonAxisInput() {
			deleteClear(_up);
			deleteClear(_right);
			deleteClear(_down);
			deleteClear(_left);
		}

		virtual Axis getAxis() const;

		void setUpAxis(std::auto_ptr<ButtonInput> button) {
			_up.push_back(button.release());
		}
		void setRightAxis(std::auto_ptr<ButtonInput> button) {
			_right.push_back(button.release());
		}
		void setDownAxis(std::auto_ptr<ButtonInput> button) {
			_down.push_back(button.release());
		}
		void setLeftAxis(std::auto_ptr<ButtonInput> button) {
			_left.push_back(button.release());
		}

	private:
		std::vector<ButtonInput*> _up, _right, _down, _left;

	};

    /// Ͻ貾ۥ饹
	class InputBase {
	public:
		virtual ~InputBase() {};

		virtual Axis getAxis() const =0;
		virtual bool getButton(int id) const =0;

		/// ޤŬ
		typedef std::map<int, bool> AllButton;
		typedef std::auto_ptr<AllButton> AllButtonPtr;
		virtual AllButtonPtr getAllButton() const;

		virtual bool getAny() const;
		virtual bool getAnyButton() const;

	protected:
		/// ǥեȤ getAllButton ȤȤϤ ID 롣
		std::vector<int> buttons_;
	};

	/// ϥ饹
	class Input : public InputBase {
	private:
		typedef std::multimap<int, ButtonInput*> ButtonMap;

	public:
		virtual ~Input() {
			for (std::map<int, ButtonInput*>::iterator ite =
					 _buttonMap.begin();
				 ite != _buttonMap.end(); ite++)
			{
				delete ite->second;
			}
			deleteClear(_axis);
		};

		void setAxis(std::auto_ptr<AxisInput> axis) {
			_axis.push_back(axis.release());
		}
		virtual Axis getAxis() const;

		void setButton(int id, std::auto_ptr<ButtonInput> button) {
			_buttonMap.insert(std::make_pair(id, button.release()));
		}
		virtual bool getButton(int id) const;

		virtual AllButtonPtr getAllButton() const;

	private:
		std::vector<AxisInput*> _axis;
		ButtonMap _buttonMap;
	};

	/// Ϥξ֤¸륯饹
	class InputState : public InputBase {
	public:
		explicit InputState(const InputBase& input);

		virtual Axis getAxis() const { return _axis; }
		virtual bool getButton(int id) const;

		virtual AllButtonPtr getAllButton() const;

	protected:
		Axis _axis;
		AllButtonPtr _buttonMap;

	};

	/// Ϸ
	class InputMgr : public Input {
	public:
		virtual ~InputMgr();

		void setDevice(std::auto_ptr<InputBase> input);

		virtual Axis getAxis() const;
		virtual bool getButton(int id) const;

		virtual AllButtonPtr getAllButton() const;

	protected:
		std::vector<InputBase*> _inputs;
	};

	/// եɤǥ饹롣
	/**
	 * @todo ʤȤҤɤǤФ
 	 */
	std::auto_ptr<Input> generateInputFromConf(const std::string& filename);

	/// ܥϤԤButtonInput
	/**
	 * SDL_QUIT 褿NULL֤
	 */
	std::auto_ptr<ButtonInput> generateAnyButtonInput();

	/// ʸ󤫤ButtonInput
	std::auto_ptr<ButtonInput>
	generateButtonInputFromString(const std::string& str);

	/// إ
	//@{
	std::auto_ptr<AxisInput> generateKeyAxis();
	std::auto_ptr<AxisInput> generateJoyAxis(int index);
	//@}

}

#endif // ! __MYSDL_PADINPUT_H__
