#include <cmath>

#include "scroll.h"
#include "util.h"
#include "game.h"

void Scroll::setPosition(double x, double y, int t) {
	if (t > endTurn_) endTurn_ = t;

	int ft = positions_.size();
	if (ft == 0) {
		ASSERT(t == 0, "<p>: first position is not set.");
		positions_.push_back(Point(x, y));
	}
	else {
		Point first(positions_.back());
		Point diff(Point(x, y) - first);
		for (int i = ft; i <= t; i++) {
			positions_.push_back(first + diff * (i-ft) / (t-ft));
		}
	}
}

void Scroll::setRoll(double d, int t) {
	if (t > endTurn_) endTurn_ = t;

	int ft = rolls_.size();
	if (ft == 0) {
		ASSERT(t == 0, "<r>: first roll is not set.");
		rolls_.push_back(d);
	}
	else {
		double first = rolls_.back();
		double diff(-d - first);
		for (int i = ft; i <= t; i++) {
			rolls_.push_back(first + diff * (i-ft) / (t-ft));
		}
	}
}

Point Scroll::convertPoint(const Point& pnt) const {
	using std::cos;
	using std::sin;

	int t = Game::instance()->turn();
	if (isEnd()) t = endTurn_;

	const Point& p = positions_[t];

	if (rolls_.empty()) {
		return convertScale(pnt - p);
	}
	else {
		double r = dtor(rolls_[t]);
		return convertScale(Point( pnt.xd()*cos(r)+pnt.yd()*sin(r) - p.xd(),
								  -pnt.xd()*sin(r)+pnt.yd()*cos(r) - p.yd()));
	}
}

Point Scroll::convertScale(const Point& pnt) const {
	return Point(pnt.xd()*172.8 , -pnt.yd()*230.4)+ Point(180, 240);
}

double Scroll::convertDir(double r) const {
	return r + dtor(rolls_[Game::instance()->turn()]);
}

bool Scroll::isEnd() const {
	return Game::instance()->turn() > endTurn_;
}
