#include "util.h"

#include <cmath>
#include <cstdlib>
#include <sstream>
#include <iostream>

const Color Color::GRAY  (127, 127, 127);
const Color Color::WHITE (255, 255, 255);
const Color Color::BLACK (  0,   0,   0);
const Color Color::RED   (255,   0,   0);
const Color Color::ORANGE(255, 127,   0);
const Color Color::YELLOW(255, 255,   0);
const Color Color::GREEN (  0, 255,   0);
const Color Color::BLUE  (  0,   0, 255);
const Color Color::PURPLE(255,   0, 255);

int rnd(int x) { return static_cast<int>(x*(double)std::rand()/RAND_MAX); }
double rnd(double x) { return x*std::rand()/RAND_MAX; }

void ASSERT(bool t, const char* str) {
    if (!t) throw std::runtime_error(std::string(str));
}

void ASSERT(bool t, const std::string& str) {
    if (!t) throw std::runtime_error(str);
}

void ignoreBrankAndComment(std::string& str, std::string delim) {
    unsigned int s;

    // Ȥν
    if ((s = str.find(delim)) != std::string::npos)
	str.erase(s);

    // Ƭν
    if (!str.empty() && str[0] == ' ')
	str.erase(0, str.find_first_not_of(' '));
}

double dtor(double x) { return x*PI/180; }
double rtod(double x) { return x*180/PI; }

Point::Point () {
}

Point::Point (const Point& rhs)
    : x(rhs.x), y(rhs.y)
{
}

Point::Point (const int x1, const int y1)
    : x(x1), y(y1)
{
}

Point::Point (const double x1, const double y1)
    : x(x1), y(y1)
{
}

Point::Point (const PolarCoord& rhs)
    : x(std::sin(rhs.r)*rhs.l), y(-std::cos(rhs.r)*rhs.l)
{
}

double Point::length() const {
    return std::sqrt(x*x+y*y);
}

double Point::length(const Point& p) const {
    double x1 = x-p.x;
    double y1 = y-p.y;
    return std::sqrt(x1*x1+y1*y1);
}

double Point::angle() const {
    double ret;
    if (y == 0) ret = PI/2;
    else ret = std::atan(x/y);
    return (y > 0) ? PI-ret : -ret;
}

double Point::angle(const Point& p) const {
    Point vec(*this - p);
    return vec.angle();
}

const Point& Point::unit() {
    double x2 = std::abs(x)*x;
    double y2 = std::abs(y)*y;
    double x2y2 = std::abs(x2)+std::abs(y2);
    x = x2 / x2y2;
    y = y2 / x2y2;

    return *this;
}

Rect::Rect () {
}

Rect::Rect (const Rect& rhs) {
    val_.x = rhs.val_.x;
    val_.y = rhs.val_.y;
    val_.w = rhs.val_.w;
    val_.h = rhs.val_.h;
}

Rect::Rect (const int xs, const int ys, const int xe, const int ye,
	    const bool to) {
    val_.x = xs;
    val_.y = ys;

    // /δط/٥ȥδط
    if (to) {
	val_.w = xe;
	val_.h = ye;
    }
    else {
	val_.w = xe - xs + 1;
	val_.h = ye - ys + 1;
    }    
}

Rect::Rect (const Point ps, const Point pe, const bool to) {
    *this = Rect(static_cast<int>(ps.x), static_cast<int>(ps.y),
		 static_cast<int>(pe.x), static_cast<int>(pe.y), to);
}

PolarCoord::PolarCoord() {}

PolarCoord::PolarCoord(const PolarCoord& rhs)
    : l(rhs.l), r(rhs.r) {}

PolarCoord::PolarCoord(double l0, double r0)
    : l(l0), r(r0) {}

PolarCoord::PolarCoord(const Point& rhs)
    : l(rhs.length()), r(rhs.angle()) {}

int str2int(const std::string& str) {
	int ret;
	std::istringstream iss(str);
	iss >> ret;
	return ret;
}

double str2double(const std::string& str) {
	double ret;
	std::istringstream iss(str);
	iss >> ret;
	return ret;
}

std::string int2string(int i) {
	std::ostringstream oss;
	oss << i;
	return oss.str();
}

std::ostream& operator<< (std::ostream& os, const Point& p) {
        os << "(" << p.xd() << "," << p.yd() << ")";
        return os;
}

std::ostream& operator<< (std::ostream& os, const Color& c) {
        os << "(" << c.r() << "," << c.g() << "," << c.b() << ")";
        return os;
}


