#include "xmltree.h"
#include "util.h"

#include <iostream>
#include <algorithm>

XMLNode::XMLNode(const std::string& name) : name_(name) {
    setReleaseDuty(true);
}

XMLNode::~XMLNode() {}

void XMLNode::dump() {
    std::cout << "<" << name_;
    AttributeMap::const_iterator ite;
    for (ite = attributes_.begin(); ite != attributes_.end(); ite++) {
	std::cout << " " << ite->first << "=" << ite->second;
    }
    std::cout << ">" << std::endl;

    if (val_ != "") std::cout << val_ << std::endl;

    std::for_each(childBegin(), childEnd(), std::mem_fun(&XMLNode::dump));

    std::cout << "</" << name_ << ">" << std::endl;
}

void XMLNode::addAttribute(const std::string& key, const std::string& val) {
    attributes_[key] = val;
}

bool XMLNode::haveAttribute(const std::string& key) const {
    return attributes_.find(key) != attributes_.end();
}

const std::string& XMLNode::getAttribute(const std::string& key) const {
    AttributeMap::const_iterator ite = attributes_.find(key);
    ASSERT(ite != attributes_.end(),
	   name_ + "hasn't key " + key + " doesn't exist");
    return ite->second;
}

XMLNode* XMLNode::getChild(const std::string& name) {
    ChildIterator ite;
    for (ite = childBegin(); ite != childEnd(); ite++) {
	if ((*ite)->getName() == name) return *ite;
    }
    return 0;
}

bool XMLNode::findNode(const std::string& name) const {
    if (getName() == name) return true;
    ConstChildIterator ite;
    for (ite = childBegin(); ite != childEnd(); ite++) {
	if ((*ite)->findNode(name)) return true;
    }
    return false;
}

XMLNode* XMLNode::next() {
    XMLNode* parent = getParent();
    if (parent == 0) return 0;
    ChildIterator ite =
	std::find(parent->childBegin(), parent->childEnd(), this);
    ASSERT(ite != parent->childEnd(), name_ + ": not found");
    ite++;
    if (ite == parent->childEnd()) return 0;
    else return *ite;
}
